import re
import datetime

market_var = {'cffex': ['IF','IC','IH','T','TF','TS'],
'dce':['C','CS','A','B','M','Y','P','FB','BB','JD','L','V','PP','J','JM','I'],
'czce':['WH','PM','CF','SR','TA','OI','RI','MA','ME','FG','RS','RM','ZC','JR','LR','SF','SM','WT','TC','GN','RO','ER','SRX','SRY','WSX','WSY','CY','AP'],
'shfe':['CU','AL','ZN','PB','NI','SN','AU','AG','RB','WR','HC','FU','BU','RU']
}

vars=[]
[vars.extend(i) for i in market_var.values()]




headers = {'Host': 'www.czce.com.cn',
           'Connection': 'keep-alive',
           'Cache-Control': 'max-age=0',
           'Accept': 'text/html, */*; q=0.01',
           'X-Requested-With': 'XMLHttpRequest',
           'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.87 Safari/537.36',
           'DNT': '1',
           'Referer': 'http://www.super-ping.com/?ping=www.google.com&locale=sc',
           'Accept-Encoding': 'gzip, deflate, sdch',
           'Accept-Language': 'zh-CN,zh;q=0.8,ja;q=0.6'
           }

SYS_SPOTPRICE_URL = 'http://www.100ppi.com/sf/day-%s.html'
SYS_SPOTPRICE_LATEST_URL = 'http://www.100ppi.com/sf/'

SHFE_VOLRANK_URL = 'http://www.shfe.com.cn/data/dailydata/kx/pm%s.dat'
CFFEX_VOLRANK_URL = 'http://www.cffex.com.cn/fzjy/ccpm/%s/%s/%s_1.csv'
DCE_VOLRANK_URL = 'http://www.dce.com.cn/publicweb/quotesdata/exportMemberDealPosiQuotesData.html?memberDealPosiQuotes.variety=%s&memberDealPosiQuotes.trade_type=0&contract.contract_id=all&contract.variety_id=%s&year=%s&month=%s&day=%s&exportFlag=txt'
CZCE_VOLRANK_URL_1 = 'http://www.czce.com.cn/cn/exchange/jyxx/pm/pm%s.html'
CZCE_VOLRANK_URL_2 = 'http://www.czce.com.cn/cn/exchange/%s/datatradeholding/%s.htm'
CZCE_VOLRANK_URL_3 = 'http://www.czce.com.cn/cn/DFSStaticFiles/Future/%s/%s/FutureDataHolding.htm'

DCE_RECIEPT_URL = 'http://www.dce.com.cn/publicweb/quotesdata/wbillWeeklyQuotes.html?wbillWeeklyQuotes.variety=all&year=%s&month=%s&day=%s'
SHFE_RECIEPT_URL_1 = 'http://www.shfe.com.cn/data/dailydata/%sdailystock.html'
SHFE_RECIEPT_URL_2 = 'http://www.shfe.com.cn/data/dailydata/%sdailystock.dat'
CZCE_RECIEPT_URL_1 = 'http://www.czce.com.cn/cn/exchange/jyxx/sheet/sheet%s.html'
CZCE_RECIEPT_URL_2 = 'http://www.czce.com.cn/cn/exchange/%s/datawhsheet/%s.htm'
CZCE_RECIEPT_URL_3 = 'http://www.czce.com.cn/cn/DFSStaticFiles/Future/%s/%s/FutureDataWhsheet.htm'

CFFEX_DAILY_URL = 'http://www.cffex.com.cn/fzjy/mrhq/%s/%s/%s_1.csv'
SHFE_DAILY_URL = 'http://www.shfe.com.cn/data/dailydata/kx/kx%s.dat'
SHFE_VWAP_URL = 'http://www.shfe.com.cn/data/dailydata/ck/%sdailyTimePrice.dat'
DCE_DAILY_URL = 'http://www.dce.com.cn//publicweb/quotesdata/dayQuotesCh.html'
CZCE_DAILY_URL_1 = 'http://www.czce.com.cn/cn/exchange/jyxx/hq/hq%s.html'
CZCE_DAILY_URL_2 = 'http://www.czce.com.cn/cn/exchange/%s/datadaily/%s.txt'
CZCE_DAILY_URL_3 = 'http://www.czce.com.cn/cn/DFSStaticFiles/Future/%s/%s/FutureDataDaily.txt'


DATE_PATTERN = re.compile(r'^([0-9]{4})[-/]?([0-9]{2})[-/]?([0-9]{2})')
FUTURE_SYMBOL_PATTERN = re.compile(r'(^[A-Za-z]{1,2})[0-9]+')


CFFEX_COLUMNS = ['open','high','low','volume','turnover','open_interest','close','settle','change1','change2']
CZCE_COLUMNS = ['pre_settle','open','high','low','close','settle','change1','change2','volume','open_interest','oi_chg','turnover','final_settle']
CZCE_COLUMNS_2 = ['pre_settle','open','high','low','close','settle','change1','volume','open_interest','oi_chg','turnover','final_settle']
SHFE_COLUMNS =  {'CLOSEPRICE': 'close',  'HIGHESTPRICE': 'high', 'LOWESTPRICE': 'low', 'OPENINTEREST': 'open_interest', 'OPENPRICE': 'open',  'PRESETTLEMENTPRICE': 'pre_settle', 'SETTLEMENTPRICE': 'settle',  'VOLUME': 'volume'}
SHFE_VWAP_COLUMNS = {':B1': 'date', 'INSTRUMENTID': 'symbol', 'TIME': 'time_range', 'REFSETTLEMENTPRICE': 'vwap'}
DCE_COLUMNS = ['open', 'high', 'low', 'close', 'pre_settle', 'settle', 'change1','change2','volume','open_interest','oi_chg','turnover']
DCE_OPTION_COLUMNS = ['open', 'high', 'low', 'close', 'pre_settle', 'settle', 'change1', 'change2', 'delta', 'volume', 'open_interest', 'oi_chg', 'turnover', 'exercise_volume']

OUTPUT_COLUMNS = ['symbol', 'date', 'open', 'high', 'low', 'close', 'volume', 'open_interest', 'turnover', 'settle', 'pre_settle', 'variety']
OPTION_OUTPUT_COLUMNS = ['symbol', 'date', 'open', 'high', 'low', 'close', 'pre_settle', 'settle', 'delta', 'volume', 'open_interest', 'oi_chg', 'turnover', 'implied_volatility', 'exercise_volume', 'variety']

DCE_MAP =  {
    '豆一': 'A',
    '豆二': 'B',
    '豆粕': 'M',
    '豆油': 'Y',
    '棕榈油': 'P',
    '玉米': 'C',
    '玉米淀粉': 'CS',
    '鸡蛋': 'JD',
    '纤维板': 'FB',
    '胶合板': 'BB',
    '聚乙烯': 'L',
    '聚氯乙烯': 'V',
    '聚丙烯': 'PP',
    '焦炭': 'J',
    '焦煤': 'JM',
    '铁矿石': 'I'
}


def convert_date(date):
    """
    transform a date string to datetime.date object.
    :param day, string, e.g. 2016-01-01, 20160101 or 2016/01/01
    :return: object of datetime.date(such as 2016-01-01) or None
    """
    if isinstance(date, datetime.date):
        return date
    elif isinstance(date, str):
        match = DATE_PATTERN.match(date)
        if match:
            groups = match.groups()
            if len(groups) == 3:
                return datetime.date(year=int(groups[0]), month=int(groups[1]), day=int(groups[2]))
    return None




# -*- coding:utf-8 -*-

import re
from fushare import cons

def symbol2varietie(symbol):
    var = ''.join(re.findall('\D',symbol)).upper().strip()
    if var == 'PTA':
        var = 'TA'
    return var

def symbolMarket(symbol):
    var = symbol2varietie(symbol)
    for (market,vars) in cons.market_var.items():
        if var in vars:
            return market

def find_chinese(string):
    p = re.compile(u'[\u4e00-\u9fa5]')
    res = re.findall(p, string)
    return ''.join(res)

def chinese_to_english(var):
    """
    翻译期货品种中文和英文缩写
    """

    chineseList=['橡胶','天然橡胶','石油沥青','沥青','沥青仓库','沥青(仓库)','沥青厂库','沥青(厂库)','热轧卷板','热轧板卷','燃料油',
                 '白银','线材','螺纹钢','铅','铜','铝','锌','黄金','钯金','锡','镍',
                 '豆一','大豆','豆二','胶合板','玉米','玉米淀粉','聚乙烯','LLDPE','LDPE','豆粕','豆油','大豆油',
                 '棕榈油','纤维板','鸡蛋','聚氯乙烯','PVC','聚丙烯','PP','焦炭','焦煤','铁矿石',
                 '强麦','强筋小麦',' 强筋小麦','硬冬白麦','普麦','硬白小麦','硬白小麦（）','皮棉','棉花','一号棉','白糖','PTA','菜籽油','菜油','早籼稻','早籼','甲醇','柴油','玻璃',
                 '油菜籽','菜籽','菜籽粕','菜粕','动力煤','粳稻','晚籼稻','晚籼','硅铁','锰硅','硬麦','棉纱','苹果',
                 '原油']
    englishList=['RU','RU','BU','BU','BU','BU','BU2','BU2','HC','HC','FU','AG','WR','RB','PB','CU','AL','ZN','AU','AU','SN','NI',
                 'A','A','B','BB','C','CS','L','L','L','M','Y','Y',
                 'P','FB','JD','V','V','PP','PP','J','JM','I',
                 'WH','WH','WH','PM','PM','PM','PM','CF','CF','CF','SR','TA','OI','OI','RI','ER','MA','MA','FG',
                 'RS','RS','RM','RM','ZC','JR','LR','LR','SF','SM','WT','CY','AP',
                 'SC']
    pos=chineseList.index(var)
    return(englishList[pos])


if __name__ == '__main__':
    print(chinese_to_english('原油'))
    symbol = 'rb1801'
    var = symbol2varietie('rb1808')
    print(var)
    market = symbolMarket(symbol)
    print(market)
    chi = find_chinese('a对方水电费dc大V')
    print(chi)


# -*- coding:utf-8 -*-

"""
Created on 2018年07月18日
@author: lowin
@contact: li783170560@126.com

从大连商品交易所、上海商品交易所、郑州商品交易所、中金交易所爬取前20会员持仓数据
建议下午16点30以后爬取当天数据，避免交易所数据更新不稳定
郑州交易所格式分为三类
"""


import requests
import json
from bs4 import BeautifulSoup
import urllib
from io import StringIO
import pandas as pd
import datetime
from fushare import cons
from fushare.symbolVar import *
from fushare.requests_fun import requests_link

rank_columns = ['vol_party_name', 'vol', 'vol_chg','long_party_name', 'long_openIntr',
   'long_openIntr_chg', 'short_party_name', 'short_openIntr', 'short_openIntr_chg']
intColumns = ['vol', 'vol_chg', 'long_openIntr', 'long_openIntr_chg', 'short_openIntr', 'short_openIntr_chg']


def get_rank_sum_daily(start=None, end=None, vars=cons.vars):
    """
        抓取四个期货交易所前5、前10、前15、前20会员持仓排名数据
        注1：由于上期所和中金所只公布每个品种内部的标的排名，没有公布品种的总排名；
            所以函数输出的品种排名是由品种中的每个标的加总获得，并不是真实的品种排名列表
        注2：大商所只公布了品种排名，未公布标的排名
        Parameters
        ------
            start: 开始日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            end: 结束数据 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            vars: 合约品种如RB、AL等列表 为空时为所有商品
        Return
        -------
            DataFrame:
                展期收益率数据(DataFrame):
                    symbol                      标的合约                     string
                    var                         商品品种                     string
                    vol_top5                    成交量前5会员成交量总和         int
                    vol_chg_top5                成交量前5会员成交量变化总和      int
                    long_openIntr_top5          持多单前5会员持多单总和         int
                    long_openIntr_chg_top5      持多单前5会员持多单变化总和      int
                    short_openIntr_top5         持空单前5会员持空单总和         int
                    short_openIntr_chg_top5     持空单前5会员持空单变化总和      int 
                    vol_top10                   成交量前10会员成交量总和        int                   
                    ...
                    
                    date                        日期                         string YYYYMMDD
    """
    start = cons.convert_date(start) if start is not None else datetime.date.today()
    end = cons.convert_date(end) if end is not None else datetime.date.today()
    records = pd.DataFrame()
    while start <= end:
        print(start)
        records = records.append(get_rank_sum(start, vars))
        start += datetime.timedelta(days=1)

    return records.reset_index(drop=True)




def get_rank_sum(date = None,vars=cons.vars):
    """
        抓取四个期货交易所前5、前10、前15、前20会员持仓排名数据
        注1：由于上期所和中金所只公布每个品种内部的标的排名，没有公布品种的总排名；
            所以函数输出的品种排名是由品种中的每个标的加总获得，并不是真实的品种排名列表
        注2：大商所只公布了品种排名，未公布标的排名
        Parameters
        ------
            date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            vars: 合约品种如RB、AL等列表 为空时为所有商品
        Return
        -------
            DataFrame:
                展期收益率数据(DataFrame):
                    symbol                      标的合约                     string
                    var                         商品品种                     string
                    vol_top5                    成交量前5会员成交量总和         int
                    vol_chg_top5                成交量前5会员成交量变化总和      int
                    long_openIntr_top5          持多单前5会员持多单总和         int
                    long_openIntr_chg_top5      持多单前5会员持多单变化总和      int
                    short_openIntr_top5         持空单前5会员持空单总和         int
                    short_openIntr_chg_top5     持空单前5会员持空单变化总和      int 
                    vol_top10                   成交量前10会员成交量总和        int                   
                    ...

                    date                        日期                         string YYYYMMDD
    """
    date = cons.convert_date(date) if date is not None else datetime.date.today()
    dce_var = [i for i in vars if i in cons.market_var['dce']]
    shfe_var = [i for i in vars if i in cons.market_var['shfe']]
    czce_var = [i for i in vars if i in cons.market_var['czce']]
    cffex_var = [i for i in vars if i in cons.market_var['cffex']]
    D={}
    if len(dce_var)>0:
        D.update(get_dce_rank_table(date, dce_var))
    if len(shfe_var)>0:
        D.update(get_shfe_rank_table(date, shfe_var))
    if len(czce_var)>0:
        D.update(get_czce_rank_table(date, czce_var))
    if len(cffex_var)>0:
        D.update(get_cffex_rank_table(date, cffex_var))
    records=pd.DataFrame()


    for symbol, table in D.items():
        table = table.applymap(lambda x: 0 if x == '' else x)
        for symbol in set(table['symbol']):

            var = symbol2varietie(symbol)
            tableCut = table[table['symbol'] == symbol]
            tableCut_top5 = tableCut[tableCut['rank'] <= 5]
            tableCut_top10 = tableCut[tableCut['rank'] <= 10]
            tableCut_top15 = tableCut[tableCut['rank'] <= 15]
            tableCut_top20 = tableCut[tableCut['rank'] <= 20]

            D = {'symbol': symbol, 'var': var,

                 'vol_top5': tableCut_top5['vol'].sum(), 'vol_chg_top5': tableCut_top5['vol_chg'].sum(),
                 'long_openIntr_top5': tableCut_top5['long_openIntr'].sum(),
                 'long_openIntr_chg_top5': tableCut_top5['long_openIntr_chg'].sum(),
                 'short_openIntr_top5': tableCut_top5['short_openIntr'].sum(),
                 'short_openIntr_chg_top5': tableCut_top5['short_openIntr_chg'].sum(),

                 'vol_top10': tableCut_top10['vol'].sum(), 'vol_chg_top10': tableCut_top10['vol_chg'].sum(),
                 'long_openIntr_top10': tableCut_top10['long_openIntr'].sum(),
                 'long_openIntr_chg_top10': tableCut_top10['long_openIntr_chg'].sum(),
                 'short_openIntr_top10': tableCut_top10['short_openIntr'].sum(),
                 'short_openIntr_chg_top10': tableCut_top10['short_openIntr_chg'].sum(),

                 'vol_top15': tableCut_top15['vol'].sum(), 'vol_chg_top15': tableCut_top15['vol_chg'].sum(),
                 'long_openIntr_top15': tableCut_top15['long_openIntr'].sum(),
                 'long_openIntr_chg_top15': tableCut_top15['long_openIntr_chg'].sum(),
                 'short_openIntr_top15': tableCut_top15['short_openIntr'].sum(),
                 'short_openIntr_chg_top15': tableCut_top15['short_openIntr_chg'].sum(),

                 'vol_top20': tableCut_top20['vol'].sum(), 'vol_chg_top20': tableCut_top20['vol_chg'].sum(),
                 'long_openIntr_top20': tableCut_top20['long_openIntr'].sum(),
                 'long_openIntr_chg_top20': tableCut_top20['long_openIntr_chg'].sum(),
                 'short_openIntr_top20': tableCut_top20['short_openIntr'].sum(),
                 'short_openIntr_chg_top20': tableCut_top20['short_openIntr_chg'].sum(),

                 'date': date.strftime('%Y%m%d')
                 }
            records = records.append(pd.DataFrame(D, index=[0]))

    if len(D.items())>0:
        add_vars = [i for i in cons.market_var['shfe']+cons.market_var['cffex'] if i in records['var'].tolist()]
        for var in add_vars:
            recordsCut = records[records['var'] == var]
            var_record = pd.DataFrame(recordsCut.sum()).T
            var_record['date'] = date.strftime('%Y%m%d')
            var_record.loc[:,['var','symbol']] = var
            records = records.append(var_record)

    return records.reset_index(drop=True)

def get_shfe_rank_table(date = None,vars = cons.vars):
    """
        抓取上海商品期货交易所前20会员持仓排名数据明细
        注：该交易所只公布每个品种内部的标的排名，没有公布品种的总排名
        Parameters
        ------
            date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            vars: 合约品种如RB、AL等列表 为空时为所有商品
            数据从20020107开始，每交易日16:30左右更新数据
        Return
        -------
            DataFrame:
                rank                        排名                        int
                vol_party_name              成交量排序的当前名次会员        string(中文)
                vol                         该会员成交量                  int
                vol_chg                     该会员成交量变化量             int
                long_party_name             持多单排序的当前名次会员        string(中文)
                long_openIntr               该会员持多单                  int
                long_openIntr_chg           该会员持多单变化量             int
                short_party_name            持空单排序的当前名次会员        string(中文)
                short_openIntr              该会员持空单                  int
                short_openIntr_chg          该会员持空单变化量             int
                symbol                      标的合约                     string
                var                         品种                        string
                date                        日期                        string YYYYMMDD
    """
    date = cons.convert_date(date) if date is not None else datetime.date.today()
    url = cons.SHFE_VOLRANK_URL %(date.strftime('%Y%m%d'))
    r = requests_link(url,'utf-8')
    try:
        context = json.loads(r.text)
    except:
        return {}
    df = pd.DataFrame(context['o_cursor'])

    df = df.rename(columns={'CJ1': 'vol', 'CJ1_CHG': 'vol_chg', 'CJ2': 'long_openIntr', 'CJ2_CHG': 'long_openIntr_chg',
                            'CJ3': 'short_openIntr',
                            'CJ3_CHG': 'short_openIntr_chg', 'PARTICIPANTABBR1': 'vol_party_name',
                            'PARTICIPANTABBR2': 'long_party_name',
                            'PARTICIPANTABBR3': 'short_party_name', 'PRODUCTNAME': 'product1', 'RANK': 'rank',
                            'INSTRUMENTID': 'symbol','PRODUCTSORTNO':'product2'})

    if len(df.columns)<3:
        return {}
    df = df.applymap(lambda x: x.strip() if type(x) == type('') else x)
    df = df.applymap(lambda x: None if x == '' else x)
    df['var'] = df['symbol'].apply(lambda x: symbol2varietie(x))

    df = df[df['rank'] > 0]
    for col in ['PARTICIPANTID1','PARTICIPANTID2','PARTICIPANTID3','product1','product2']:
        try:
            del df[col]
        except:
            pass
    get_vars = [var for var in vars if var in df['var'].tolist()]
    D={}
    for var in get_vars:
        df_var = df[df['var'] == var]
        for symbol in set(df_var['symbol']):
            df_symbol = df_var[df_var['symbol'] == symbol]
            D[symbol] = df_symbol.reset_index(drop=True)
    return D



def _czce_df_read(url,skiprow,encode='utf-8'):
    """
        抓取郑州商品期货交易所的网页数据
        Parameters
        ------
            url:        网站         string
            skiprow:    去掉前几行    int
        Return
        -------
            DataFrame
                
    """
    r = requests_link(url,encode)
    data = pd.read_html(r.text, match='.+', flavor=None, header=0, index_col=0, skiprows=skiprow, attrs=None,
                        parse_dates=False, tupleize_cols=False, thousands=', ', encoding="gbk", decimal='.',
                        converters=None, na_values=None, keep_default_na=True)
    return data

def get_czce_rank_table(date = None,vars = cons.vars):
    """
        抓取郑州商品期货交易所前20会员持仓排名数据明细
        注：该交易所即公布了品种排名，也公布了标的排名
        Parameters
        ------
            date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            vars: 合约品种如RB、AL等列表 为空时为所有商品
            数据从20050509开始，每交易日16:30左右更新数据
        Return
        -------
            DataFrame:
                rank                        排名                        int
                vol_party_name              成交量排序的当前名次会员        string(中文)
                vol                         该会员成交量                  int
                vol_chg                     该会员成交量变化量             int
                long_party_name             持多单排序的当前名次会员        string(中文)
                long_openIntr               该会员持多单                  int
                long_openIntr_chg           该会员持多单变化量             int
                short_party_name            持空单排序的当前名次会员        string(中文)
                short_openIntr              该会员持空单                  int
                short_openIntr_chg          该会员持空单变化量             int
                symbol                      标的合约                     string
                var                         品种                        string
                date                        日期                        string YYYYMMDD
    """
    date = cons.convert_date(date) if date is not None else datetime.date.today()
    if date <= datetime.date(2010, 8, 25):
        url = cons.CZCE_VOLRANK_URL_1 % (date.strftime('%Y%m%d'))
        data = _czce_df_read(url,skiprow=0)
        r = requests.get(url)
        r.encoding = 'utf-8'
        soup = BeautifulSoup(r.text, 'lxml', from_encoding="gb2312")
        symbols=[]
        for link in soup.find_all('b'):
            strings = (str(link).split(' '))
            if len(strings)>5:
                try:
                    symbol = chinese_to_english(strings[4])
                except:
                    symbol = strings[4]
                symbols.append(symbol)
        D={}
        for i in range(len(symbols)):
            symbol = symbols[i]
            tableCut = data[i+1]
            tableCut.columns = rank_columns
            tableCut = tableCut.iloc[:-1,:]
            tableCut.loc[:,'rank'] = tableCut.index
            tableCut.loc['合计','rank'] = 999
            tableCut.loc['合计',['vol_party_name','long_party_name','short_party_name']] = None
            tableCut.loc[:,'symbol'] = symbol
            tableCut.loc[:,'var'] = symbol2varietie(symbol)
            D[symbol] = tableCut.reset_index(drop=True)
        return D

    elif date <= datetime.date(2015, 11, 11):
        url = cons.CZCE_VOLRANK_URL_2 % (date.year, date.strftime('%Y%m%d'))
        data = _czce_df_read(url,skiprow=1)[1]
    elif date < datetime.date(2017, 12, 28):
        url = cons.CZCE_VOLRANK_URL_3 % (date.year, date.strftime('%Y%m%d'))
        data = _czce_df_read(url,skiprow=1)[0]
    else:
        url = cons.CZCE_VOLRANK_URL_3 % (date.year, date.strftime('%Y%m%d'))
        data = _czce_df_read(url, skiprow=0)[0]

    print(url)
    if len(data.columns) <6:
        return {}

    print(data)
    table = data.iloc[:, :9]
    table.columns = rank_columns
    table.loc[:,'rank'] = table.index
    table[intColumns] = table[intColumns].astype(str)
    table[intColumns] = table[intColumns].applymap(lambda x: x.replace(',', ''))
    table = table.applymap(lambda x: 0 if x == '-' else x)
    indexs = [i for i in table.index if '合约' in i or '品种' in i]
    indexs.insert(0,0)
    D = {}

    for i in range(len(indexs)):
        if indexs[i] == 0:
            tableCut = table.loc[:indexs[i + 1], :]
            string = tableCut.index.name

        elif i < len(indexs) - 1:
            tableCut = table.loc[indexs[i]:indexs[i + 1], :]
            string = tableCut.index[0]
        else:
            tableCut = table.loc[indexs[i]:, :]
            string = tableCut.index[0]

        if 'PTA' in string:
            symbol ='TA'
        else:
            try:
                symbol = chinese_to_english(find_chinese(re.compile('：(.*) ').findall(string)[0]))
            except:
                symbol = re.compile('：(.*) ').findall(string)[0]
        var = symbol2varietie(symbol)
        if var in vars:
            tableCut = tableCut.dropna(how='any').iloc[1:, :]
            tableCut = tableCut.loc[[x for x in tableCut.index if x in [str(i) for i in range(21)]], :]
            tableCut = _tableCut_cal(tableCut, symbol)
            D[symbol] = tableCut.reset_index(drop=True)
    return D


def get_dce_rank_table(date = None,vars = cons.vars):
    """
        抓取大连商品期货交易所前20会员持仓排名数据明细
        注：该交易所即公布了品种排名，也公布了标的排名
        Parameters
        ------
            date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            vars: 合约品种如RB、AL等列表 为空时为所有商品
            数据从20060104开始，每交易日16:30左右更新数据
        Return
        -------
            DataFrame:
                rank                        排名                        int
                vol_party_name              成交量排序的当前名次会员        string(中文)
                vol                         该会员成交量                  int
                vol_chg                     该会员成交量变化量             int
                long_party_name             持多单排序的当前名次会员        string(中文)
                long_openIntr               该会员持多单                  int
                long_openIntr_chg           该会员持多单变化量             int
                short_party_name            持空单排序的当前名次会员        string(中文)
                short_openIntr              该会员持空单                  int
                short_openIntr_chg          该会员持空单变化量             int
                symbol                      标的合约                     string
                var                         品种                        string
                date                        日期                        string YYYYMMDD
    """
	
    date = cons.convert_date(date) if date is not None else datetime.date.today()
    vars = [i for i in vars if i in cons.market_var['dce']]
    D={}
    for var in vars:
        url = cons.DCE_VOLRANK_URL % (var.lower(), var.lower(), date.year, date.month - 1, date.day)

        list_60_name = []
        list_60 = []
        list_60_chg = []
        rank = []
        texts = urllib.request.urlopen(url).readlines()
        if len(texts)>30:
            for text in texts:
                line = text.decode('utf8')
                stringlist = line.split()
                try:
                    if int(stringlist[0]) <= 20:
                        list_60_name.append(stringlist[1])
                        list_60.append(stringlist[2])
                        list_60_chg.append(stringlist[3])
                        rank.append(stringlist[0])
                except:
                    pass
            tableCut = pd.DataFrame({'rank': rank[0:20],
                               'vol_party_name': list_60_name[0:20],
                               'vol': list_60[0:20],
                               'vol_chg': list_60_chg[0:20],
                               'long_party_name': list_60_name[20:40],
                               'long_openIntr': list_60[20:40],
                               'long_openIntr_chg': list_60_chg[20:40],
                               'short_party_name': list_60_name[40:60],
                               'short_openIntr': list_60[40:60],
                               'short_openIntr_chg': list_60_chg[40:60]
                               })
            tableCut = tableCut.applymap(lambda x: x.replace(',', ''))
            tableCut = _tableCut_cal(tableCut, var)
            D[var] = tableCut
    return D

def get_cffex_rank_table(date = None,vars = cons.vars):
    """
        抓取郑州商品期货交易所前20会员持仓排名数据明细
        注：该交易所即公布了品种排名，也公布了标的排名
        Parameters
        ------
            date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            vars: 合约品种如RB、AL等列表 为空时为所有商品
            数据从20100416开始，每交易日16:30左右更新数据
        Return
        -------
            DataFrame:
                rank                        排名                        int
                vol_party_name              成交量排序的当前名次会员        string(中文)
                vol                         该会员成交量                  int
                vol_chg                     该会员成交量变化量             int
                long_party_name             持多单排序的当前名次会员        string(中文)
                long_openIntr               该会员持多单                  int
                long_openIntr_chg           该会员持多单变化量             int
                short_party_name            持空单排序的当前名次会员        string(中文)
                short_openIntr              该会员持空单                  int
                short_openIntr_chg          该会员持空单变化量             int
                symbol                      标的合约                     string
                var                         品种                        string
                date                        日期                        string YYYYMMDD
    """
    vars = [i for i in vars if i in cons.market_var['cffex']]
    date = cons.convert_date(date) if date is not None else datetime.date.today()
    D={}
    for var in vars:
        url = cons.CFFEX_VOLRANK_URL % (date.strftime('%Y%m'), date.strftime('%d'), var)
        r = requests_link(url,encoding='gbk')
        if '网页错误' not in r.text:
            table = pd.read_csv(StringIO(r.text.split('\n交易日,')[1]))
            table = table.dropna(how='any')
            table = table.applymap(lambda x: x.strip() if type(x)==type('') else x)
            for symbol in set(table['合约']):
                tableCut =table[table['合约'] == symbol]
                tableCut.columns = ['symbol','rank']+rank_columns
                tableCut = _tableCut_cal(tableCut,symbol)
                D[symbol] = tableCut.reset_index(drop=True)
    return D

def _tableCut_cal(tableCut, symbol):
    var = symbol2varietie(symbol)
    tableCut[intColumns+['rank']] = tableCut[intColumns+['rank']].astype(int)
    tableCut_sum = tableCut.sum()
    tableCut_sum['rank'] = 999
    for col in ['vol_party_name', 'long_party_name', 'short_party_name']:
        tableCut_sum[col] = None
    tableCut = tableCut.append(pd.DataFrame(tableCut_sum).T)
    tableCut['symbol'] = symbol
    tableCut['var'] = var
    return tableCut


if __name__ == '__main__':
    df = get_czce_rank_table('20180706')
    print(df)


# -*- coding:utf-8 -*- 

__version__ = '1.1.3'
__author__ = 'Lowin'


"""
大宗商品现货价格及基差
"""
from fushare.basis import (get_spotPrice_daily,
                           get_spotPrice)


"""
期货持仓成交排名数据
"""
from fushare.cot import (get_rank_sum_daily,
                         get_rank_sum,
                         get_shfe_rank_table,
                         get_czce_rank_table,
                         get_dce_rank_table,
                         get_cffex_rank_table)


"""
大宗商品仓单数据
"""
from fushare.receipt import (get_reciept)


"""
大宗商品仓单数据
"""
from fushare.rollYield import (get_rollYield_bar, get_rollYield)


"""
交易所行情数据日线
"""
from fushare.dailyBar import (get_cffex_daily,
                              get_czce_daily,
                              get_shfe_vwap,
                              get_shfe_daily,
                              get_dce_daily,
                              get_future_daily)


# -*- coding:utf-8 -*-

"""
Created on 2018年07月11日
@author: lowin
@contact: li783170560@126.com

获取各合约展期收益率，日线数据从dailyBar脚本获取
"""


import matplotlib.pyplot as plt
import pandas as pd
import datetime
from fushare import cons
from fushare.symbolVar import *
from fushare.dailyBar import *

def _plot_bar(values,xtick):
    fig = plt.figure(1)
    ax = fig.add_subplot(111)
    ax.bar(range(len(values)), values, color="green")
    ax.set_xticks(range(len(xtick)))
    ax.set_xticklabels(xtick, fontsize=6)
    plt.show()


def _plot(values, xtick):
    plt.plot(values,xtick)
    plt.show()


def get_rollYield_bar(type = 'symbol',  var = 'RB',date= None, start = None, end = None, plot = False):
    """
        获取展期收益率
    Parameters
    ------
        type = 'symbol'：获取某天某品种所有交割月合约的收盘价
        type = 'var'：获取某天所有品种两个主力合约的展期收益率（展期收益率横截面）
        type = ‘date’：获取某品种每天的两个主力合约的展期收益率（展期收益率时间序列）
        start: 开始日期 format：YYYYMMDD
        end: 结束数据 format：YYYYMMDD
        date: 某一天日期 format： YYYYMMDD
        var: 合约品种如RB、AL等
    Return
    -------
        DataFrame
            展期收益率数据(DataFrame):
                ry      展期收益率
                index   日期或品种
    """

    date = cons.convert_date(date) if date is not None else datetime.date.today()
    start = cons.convert_date(start) if start is not None else datetime.date.today()
    end = cons.convert_date(end) if end is not None else datetime.date.today()
	
    if type == 'symbol':
        df = get_future_daily(start=date, end=date, market=symbolMarket(var))
        df = df[df['variety'] == var]
        if plot:
            _plot_bar(df['close'].tolist(), df['symbol'].tolist())
        return df

    if type == 'var':
        df = pd.DataFrame()
        for market in ['dce','cffex','shfe','czce']:
            df = df.append(get_future_daily(start=date, end=date, market=market))
        varList = list(set(df['variety']))
        ryList = []
        for var in varList:
            ryList.append(get_rollYield(date, var, df=df))
        df = pd.DataFrame(ryList,index = varList,columns = ['ry'])
        df = df.sort_values('ry')
        if plot:
            _plot_bar(df['ry'].tolist(), df.index)
        return df

    if type == 'date':
        dfL=pd.DataFrame()
        while start <= end:
            try:
                ry = get_rollYield(start, var)
                dfL = dfL.append(pd.DataFrame([ry], index=[start], columns=['ry']))
            except:
                pass
            start += datetime.timedelta(days=1)
        if plot:
            _plot(pd.to_datetime(dfL.index), dfL['ry'].tolist())
        return dfL



def get_rollYield(date = None, var = 'IF',symbol1 = None, symbol2 = None, df = None):
    """
            获取某一天某一品种（主力和次主力）、或固定两个合约的展期收益率
        Parameters
        ------
            date: string 某一天日期 format： YYYYMMDD
            var: string 合约品种如RB、AL等            
            symbol1: string 合约1如rb1810
            symbol2: string 合约2如rb1812
            df: DataFrame或None 从dailyBar得到合约价格，如果为空就在函数内部抓dailyBar，直接喂给数据可以让计算加快
        Return
        -------
            DataFrame
                展期收益率数据(DataFrame):
                    ry      展期收益率
                    index   日期或品种
    """
    date = cons.convert_date(date) if date is not None else datetime.date.today()
    if symbol1:
        var = symbol2varietie(symbol1)
    if type(df) != type(pd.DataFrame()):
        market = symbolMarket(var)
        df = get_future_daily(start=date, end=date, market=market)
    if var:
        df = df[df['variety'] == var].sort_values('open_interest',ascending=False)
        df['close']=df['close'].astype('float')
        priceRate = df['close'].pct_change().tolist()[1]
        symbol1 = df['symbol'].tolist()[0]
        symbol2 = df['symbol'].tolist()[1]
    else:
        close1 = df['close'][df['symbol'] == symbol1.upper()].tolist()[0]
        close2 = df['close'][df['symbol'] == symbol2.upper()].tolist()[0]
        priceRate = close2/close1-1
    mc = _monthChange(symbol2, symbol1)
    ry = priceRate/mc
    return ry

def _monthChange(symbol1,symbol2):
    """
            获取两个合约的交割月的月间差
        Parameters
        ------          
            symbol1: string 合约1如rb1810
            symbol2: string 合约2如rb1812
        Return
        -------
            c： int 月间差
    """

    A = re.sub(r'\D', '', symbol1)
    A1 = int(A[:-2])
    A2 = int(A[-2:])
    B = re.sub(r'\D', '', symbol2)
    B1 = int(B[:-2])
    B2 = int(B[-2:])
    c = (A1 - B1) * 12 + (A2 - B2)
    return c

if __name__ == '__main__':

    d = get_rollYield_bar(type='var',date='20180710')
    print(d)



# -*- coding:utf-8 -*-

"""
Created on 2018年07月18日
@author: lowin
@contact: li783170560@126.com

用requests包爬取网站内容，在链接失败后可重复爬取
"""

import requests
import time
def requests_link(url,encoding='utf-8'):
    """
            爬取网站内容，如网站链接失败，可重复爬取20次
            Parameters
            ------
                url: 网站 string
                encoding: 编码类型 string：’utf-8‘、’gbk‘等
            Return
            -------
                r： 爬取返回内容 response: 

        """
    i=0
    while True:
        try:
            r = requests.get(url,timeout = 5)
            r.encoding = encoding
            return r
        except:
            i+=1
            print('第%s次链接失败' %str(i))
            time.sleep(5)
            if i>20:
                return None



# -*- coding:utf-8 -*-
'''
Created on 2018年07月12日
@author: lowin
@contact: li783170560@126.com

从生意社网站爬取大宗商品现货价格，及相应基差
网站数据含有20110104至今
'''



import requests
import re
import pandas as pd
import datetime
import time
from fushare import cons
from fushare.symbolVar import *

def get_spotPrice_daily(start = None, end = None, vars = cons.vars):
    """
        获取大宗商品现货价格，及相应基差
    Parameters
    ------
        start: 开始日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
        end: 结束数据 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
        vars: 合约品种如RB、AL等列表 为空时为所有商品
    Return
    -------
        DataFrame
            展期收益率数据(DataFrame):
                var             商品品种                     string
                SP              现货价格                     float
                nearSymbol      临近交割合约                  string
                nearPrice       临近交割合约结算价             float
                domSymbol       主力合约                     string
                domPrice        主力合约结算价                float
                nearBasis       临近交割合约相对现货的基差       float
                domBasis        主力合约相对现货的基差          float
                nearBasisRate   临近交割合约相对现货的基差率     float
                domBasisRate    主力合约相对现货的基差率        float
                date            日期                        string YYYYMMDD
    """

    start = cons.convert_date(start) if start is not None else datetime.date.today()
    end = cons.convert_date(end) if end is not None else datetime.date.today()
    df_list=[]
    while start <= end:
        print(start)
        df = get_spotPrice(start,vars)
        if df is not None:
            df_list.append(df)
        start += datetime.timedelta(days = 1)

    if len(df_list) > 0:
        return pd.concat(df_list).reset_index(drop=True)


def get_spotPrice(date = None,vars = cons.vars):
    """
        获取某一天大宗商品现货价格，及相应基差
    Parameters
    ------
        date: 开始日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
        vars: 合约品种如RB、AL等列表 为空时为所有商品
    Return
    -------
        DataFrame
            展期收益率数据(DataFrame):
                var             商品品种                     string
                SP              现货价格                     float
                nearSymbol      临近交割合约                  string
                nearPrice       临近交割合约结算价             float
                domSymbol       主力合约                     string
                domPrice        主力合约结算价                float
                nearBasis       临近交割合约相对现货的基差      float
                domBasis        主力合约相对现货的基差         float
                nearBasisRate   临近交割合约相对现货的基差率    float
                domBasisRate    主力合约相对现货的基差率       float
                date            日期                       string YYYYMMDD
    """
	
    date = cons.convert_date(date) if date is not None else datetime.date.today()
    u1 = cons.SYS_SPOTPRICE_LATEST_URL
    u2 = cons.SYS_SPOTPRICE_URL %date.strftime('%Y-%m-%d')
    i = 1
    while True:
        for url in [u2,u1]:
            try:
                r=requests.get(url,timeout=2)
                string = pd.read_html(r.text)[0].loc[1,1]
                news = ''.join(re.findall(r'[0-9]',string))
                if news[3:11] == date.strftime('%Y%m%d'):
                    records = _check_information(pd.read_html(r.text)[1],date)
                    records.index = records['var']
                    vars_inMarket = [i for i in vars if i in records.index]
                    return records.loc[vars_inMarket,:].reset_index(drop=True)
                else:
                    time.sleep(3)
                    return pd.DataFrame()
            except Exception as e:
                print('%s日生意社数据连接失败，第%s次尝试，最多5次' % (date.strftime('%Y-%m-%d'), str(i)))
                i+=1
                if i > 5:
                    return pd.DataFrame()


def _check_information(df, date):
    df = df.loc[:, [0, 1, 2, 3, 7, 8]]
    df.columns = ['var', 'SP', 'nearSymbol', 'nearPrice', 'domSymbol', 'domPrice']
    records=pd.DataFrame()
    for string in df['var'].tolist():
        if string == 'PTA':
            news = 'PTA'
        else:
            news = ''.join(re.findall(r'[\u4e00-\u9fa5]', string))
        if news != '' and news not in ['商品', '价格', '上海期货交易所', '郑州商品交易所', '大连商品交易所']:
            var = chinese_to_english(news)
            record = df[df['var'] == string]
            record.loc[:,'var'] = var
            record.loc[:,'SP'] = record.loc[:,'SP'].astype(float)
            if var == 'JD':
                record.loc[:,'SP'] = float(record['SP']) * 500
            if var == 'FG':
                record.loc[:,'SP'] = record['SP'] * 80

            records = records.append(record)


    records.loc[:, ['nearPrice', 'domPrice', 'SP']] = records.loc[:, ['nearPrice', 'domPrice', 'SP']].astype(
        'float')

    records.loc[:, 'nearSymbol'] = records['nearSymbol'].replace('[^0-9]*(\d*)$', '\g<1>', regex=True)
    records.loc[:, 'domSymbol'] = records['domSymbol'].replace('[^0-9]*(\d*)$', '\g<1>', regex=True)

    records.loc[:, 'nearSymbol'] = records['var'] + records.loc[:, 'nearSymbol'].astype('int').astype('str')
    records.loc[:, 'domSymbol'] = records['var'] + records.loc[:, 'domSymbol'].astype('int').astype('str')

    records['nearSymbol'] = records['nearSymbol'].apply(lambda x: x.lower() if x[:-4] in cons.market_var['shfe']+cons.market_var['dce'] else x)
    records.loc[:,'domSymbol'] = records.loc[:,'domSymbol'].apply(lambda x: x.lower() if x[:-4] in cons.market_var['shfe']+cons.market_var['dce'] else x)
    records.loc[:,'nearSymbol'] = records.loc[:,'nearSymbol'].apply(lambda x: x[:-4]+x[-3:] if x[:-4] in cons.market_var['czce'] else x)
    records.loc[:,'domSymbol'] = records.loc[:,'domSymbol'].apply(lambda x: x[:-4]+x[-3:] if x[:-4] in cons.market_var['czce'] else x)

    records['nearBasis'] = records['nearPrice'] - records['SP']
    records['domBasis'] = records['domPrice'] - records['SP']
    records['nearBasisRate'] = records['nearPrice']/records['SP']-1
    records['domBasisRate'] = records['domPrice']/records['SP']-1
    records.loc[:, 'date'] = date.strftime('%Y%m%d')
    return records




if __name__ == '__main__':
    df = get_spotPrice_daily(start ='20180901', end ='20180910',vars = ['TA'])
    print(df)


# -*- coding:utf-8 -*-

"""
Created on 2018年07月12日
@author: lowin
@contact: li783170560@126.com

从大连商品交易所、上海商品交易所、郑州商品交易所爬取每日仓单数据
建议下午16点30以后爬取当天数据，避免交易所数据更新不稳定

"""

import requests
import json
import re
import pandas as pd
import datetime
from fushare import cons
from fushare.symbolVar import *

shfe_20100126 = pd.DataFrame({'var':['CU','AL','ZN','RU','FU','AU','RB','WR'],'reciept':[29783,285396,187713,116435,376200,12,145648,0]})
shfe_20101029 = pd.DataFrame({'var':['CU','AL','ZN','RU','FU','AU','RB','WR'],'reciept':[39214,359729,182562,25990,313600,27,36789,0]})
# ----------------------------------------------------------------------
def get_dce_reciept(date = None,vars=cons.vars):
    """
        抓取大连商品交易所注册仓单数据
        Parameters
        ------
            date: 开始日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            vars: 合约品种如RB、AL等列表 为空时为所有商品
            数据从20060106开始，每周五更新仓单数据。直到20090407起，每交易日都更新仓单数据
        Return
        -------
            DataFrame:
                展期收益率数据(DataFrame):
                    var             商品品种                     string
                    reciept         仓单数                       int
                    date            日期                         string YYYYMMDD
    """
    date = cons.convert_date(date) if date is not None else datetime.date.today()
    url = cons.DCE_RECIEPT_URL % (date.year, date.month - 1, date.day)
    data = pd.read_html(url, encoding='utf-8')[0]
    records=pd.DataFrame()
    for x in data.to_dict(orient='records'):
        if type(x[0]) == type('a'):
            if x[0][-2:] == '小计':
                var = x[0][:-2]
                D = {'var':chinese_to_english(var),'reciept':int(x[3]),'date':date.strftime('%Y%m%d')}
                records = records.append(pd.DataFrame(D,index=[0]))
    if len(records.index) != 0:
        records.index = records['var']
        vars_inMarket = [i for i in vars if i in records.index]
        records = records.loc[vars_inMarket, :]
    return records.reset_index(drop=True)

# ----------------------------------------------------------------------
def get_shfe_reciept_1(date = None,vars = cons.vars):
    """
        抓取上海商品交易所注册仓单数据
        适用20081006至20140518(包括)
        20100126、20101029日期交易所格式混乱，直接回复脚本中DataFrame
        20100416、20130821日期交易所数据丢失
        Parameters
        ------
            date: 开始日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            vars: 合约品种如RB、AL等列表 为空时为所有商品
        Return
        -------
            DataFrame:
                展期收益率数据(DataFrame):
                    var             商品品种                     string
                    reciept         仓单数                       int
                    date            日期                         string YYYYMMDD
    """
    date = cons.convert_date(date).strftime('%Y%m%d') if date is not None else datetime.date.today()
    if date == '20100126':
        shfe_20100126['date']=date
        return shfe_20100126
    elif date == '20101029':
        shfe_20101029['date'] = date
        return shfe_20101029
    else:
        varList = ['天然橡胶', '沥青仓库', '沥青厂库', '热轧卷板', '燃料油', '白银', '线材', '螺纹钢', '铅', '铜', '铝', '锌', '黄金', '锡', '镍']
        url = cons.SHFE_RECIEPT_URL_1 % date
        try:
            data = pd.read_html(url)[0]
        except:
            return pd.DataFrame()
        indexs = [x for x in data.index if (data[0].tolist()[x] in varList)]
        lastIndex = [x for x in data.index if '注' in str(data[0].tolist()[x])][0]-1
        records = pd.DataFrame()
        for i in list(range(len(indexs))):
            if i !=len(indexs)-1:
                dataCut = data.loc[indexs[i]:indexs[i+1]-1,:]
            else:
                dataCut = data.loc[indexs[i]:lastIndex,:]
                dataCut = dataCut.fillna(method='pad')
            D={}
            D['var'] = chinese_to_english(dataCut[0].tolist()[0])
            D['reciept'] = dataCut[1].tolist()[-1]
            D['date'] = date
            records = records.append(pd.DataFrame(D,index=[0]))
    if len(records.index) != 0:
        records.index = records['var']
        vars_inMarket = [i for i in vars if i in records.index]
        records = records.loc[vars_inMarket, :]
    return records.reset_index(drop=True)



# ----------------------------------------------------------------------
def get_shfe_reciept_2(date = None,vars=None):
    """
        抓取上海商品交易所注册仓单数据
        适用20140519(包括)至今
        Parameters
        ------
            date: 开始日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            vars: 合约品种如RB、AL等列表 为空时为所有商品
        Return
        -------
            DataFrame:
                展期收益率数据(DataFrame):
                    var             商品品种                     string
                    reciept         仓单数                       int
                    date            日期                         string YYYYMMDD
    """
    date = cons.convert_date(date).strftime('%Y%m%d') if date is not None else datetime.date.today()
    url = cons.SHFE_RECIEPT_URL_2 % date
    r = requests.get(url)
    r.encoding = 'utf-8'
    try:
        context = json.loads(r.text)
    except:
        return pd.DataFrame()
    data = pd.DataFrame(context['o_cursor'])
    if len(data.columns) <1:
        return pd.DataFrame()
    records = pd.DataFrame()
    for var in set(data['VARNAME'].tolist()):
        dataCut = data[data['VARNAME'] == var]
        D = {'var':chinese_to_english(re.sub("\W|[a-zA-Z]", "", var)),'reciept':dataCut['WRTWGHTS'].tolist()[-1],'date':date}
        records = records.append(pd.DataFrame(D,index=[0]))
    if len(records.index) != 0:
        records.index = records['var']
        vars_inMarket = [i for i in vars if i in records.index]
        records = records.loc[vars_inMarket, :]
    return records.reset_index(drop=True)

# ----------------------------------------------------------------------
def get_czce_reciept_1(date = None, vars=cons.vars):
    """
        抓取郑州商品交易所注册仓单数据
        适用20080222至20100824(包括)
        Parameters
        ------
            date: 开始日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            vars: 合约品种如CF、TA等列表 为空时为所有商品
        Return
        -------
            DataFrame:
                展期收益率数据(DataFrame):
                    var             商品品种                     string
                    reciept         仓单数                       int
                    date            日期                         string YYYYMMDD
    """
    date = cons.convert_date(date).strftime('%Y%m%d') if date is not None else datetime.date.today()
    url = cons.CZCE_RECIEPT_URL_1 % date
    print(url)
    r = requests.get(url)
    r.encoding = 'utf-8'
    context = r.text
    data = pd.read_html(context)[1]
    records=pd.DataFrame()
    indexs= [x for x in data.index if '品种：' in str(data[0].tolist()[x])]
    for i in list(range(len(indexs))):
        if i != len(indexs) - 1:
            dataCut = data.loc[indexs[i]:indexs[i + 1] - 1, :]
            dataCut = dataCut.fillna(method='pad')
        else:
            dataCut = data.loc[indexs[i]:, :]
            dataCut = dataCut.fillna(method='pad')
        if 'PTA' in dataCut[0].tolist()[0]:
            var = 'TA'
        else:
            var = chinese_to_english(re.sub('[A-Z]+', '', dataCut[0].tolist()[0][3:]))
        if var == 'CF':
            reciept = dataCut[6].tolist()[-1]
        else:
            reciept =dataCut[5].tolist()[-1]
        D = {'var':var, 'reciept':reciept, 'date':date}
        records = records.append(pd.DataFrame(D,index=[0]))
    if len(records.index) != 0:
        records.index = records['var']
        vars_inMarket = [i for i in vars if i in records.index]
        records = records.loc[vars_inMarket, :]
    return records.reset_index(drop=True)
# ----------------------------------------------------------------------
def get_czce_reciept_2(date = None,vars = cons.vars):
    """
        抓取郑州商品交易所注册仓单数据
        适用20100825(包括)至20151111(包括)
        Parameters
        ------
            date: 开始日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            vars: 合约品种如CF、TA等列表 为空时为所有商品
        Return
        -------
            DataFrame:
                展期收益率数据(DataFrame):
                    var             商品品种                     string
                    reciept         仓单数                       int
                    date            日期                         string YYYYMMDD
    """
    date = cons.convert_date(date).strftime('%Y%m%d') if date is not None else datetime.date.today()
    url = cons.CZCE_RECIEPT_URL_2 % (date[:4], date)
    r = requests.get(url)
    r.encoding = 'utf-8'
    data = pd.read_html(r.text)[3:]
    records=pd.DataFrame()
    for dataCut in data:
        if len(dataCut.columns)>3:
            lastIndexs = [x for x in dataCut.index if '注：' in str(dataCut[0].tolist()[x])]
            if len(lastIndexs)>0:
                lastIndex = lastIndexs[0] - 1
                dataCut = dataCut.loc[:lastIndex,:]
            if 'PTA' in dataCut[0].tolist()[0]:
                var = 'TA'
            else:
                strings = dataCut[0].tolist()[0]
                string = strings.split(' ')[0][3:]
                var = chinese_to_english(re.sub('[A-Z]+', '', string))
            reciept = dataCut.iloc[:,-3].tolist()[-1]
            D = {'var':var, 'reciept':reciept, 'date':date}
            records = records.append(pd.DataFrame(D,index=[0]))
    if len(records.index) != 0:
        records.index = records['var']
        vars_inMarket = [i for i in vars if i in records.index]
        records = records.loc[vars_inMarket, :]
    return records.reset_index(drop=True)
# ----------------------------------------------------------------------
def get_czce_reciept_3(date = None, vars = cons.vars):
    """
        抓取郑州商品交易所注册仓单数据
        适用20151112(包括)至今
        Parameters
        ------
            date: 开始日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            vars: 合约品种如CF、TA等列表 为空时为所有商品
        Return
        -------
            DataFrame:
                展期收益率数据(DataFrame):`1
                    var             商品品种                     string
                    reciept         仓单数                       int
                    date            日期                         string YYYYMMDD
    """
    date = cons.convert_date(date).strftime('%Y%m%d') if date is not None else datetime.date.today()
    url = cons.CZCE_RECIEPT_URL_3 % (date[:4], date)
    r = requests.get(url)
    r.encoding = 'utf-8'
    data = pd.read_html(r.text, encoding='gb2312')
    records=pd.DataFrame()
    if len(data) < 4:
        return records
    if int(date) <= 20171227:
        data = data[1:]
    for dataCut in data:
        if len(dataCut.columns) > 3:
            lastIndexs = [x for x in dataCut.index if '注：' in str(dataCut[0].tolist()[x])]
            if len(lastIndexs) > 0:
                lastIndex = lastIndexs[0] - 1
                dataCut = dataCut.loc[:lastIndex, :]
            if 'PTA' in dataCut[0].tolist()[0]:
                var = 'TA'
            else:
                strings = dataCut[0].tolist()[0]
                string = strings.split(' ')[0][3:]
                var = chinese_to_english(re.sub('[A-Z]+', '', string))
            dataCut.columns = dataCut.loc[1,:]
            dataCut = dataCut.fillna(method='pad')
            try:
                reciept = dataCut.loc[:, '仓单数量'].tolist()[-1]
            except:
                reciept = dataCut.loc[:, '仓单数量(保税)'].tolist()[-1]
            D = {'var': var, 'reciept': reciept, 'date': date}
            records = records.append(pd.DataFrame(D, index=[0]))
    if len(records.index) != 0:
        records.index = records['var']
        vars_inMarket = [i for i in vars if i in records.index]
        records = records.loc[vars_inMarket, :]
    return records.reset_index(drop=True)

# ----------------------------------------------------------------------
def get_reciept(start=None, end=None, vars=cons.vars):
    """
        获取大宗商品注册仓单数量
        Parameters
        ------
            start: 开始日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            end: 结束数据 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
            vars: 合约品种如RB、AL等列表 为空时为所有商品
        Return
        -------
            DataFrame
                展期收益率数据(DataFrame):
                    var             商品品种                     string
                    reciept         仓单数量                     int
                    date            日期                        string YYYYMMDD
    """
    start = cons.convert_date(start) if start is not None else datetime.date.today()
    end = cons.convert_date(end) if end is not None else datetime.date.today()
    records=pd.DataFrame()
    while start <= end:
        print(start)
        for market,marketVars in cons.market_var.items():

            if market == 'dce':
                f = get_dce_reciept
            elif market == 'shfe':
                if start <= datetime.date(2014,5,16):
                    f = get_shfe_reciept_1
                else:
                    f = get_shfe_reciept_2
            elif market == 'czce':
                if start <= datetime.date(2010,8,24):
                    f = get_czce_reciept_1
                elif start <= datetime.date(2015,11,11):
                    f = get_czce_reciept_2
                else:
                    f = get_czce_reciept_3


            get_vars = [var for var in vars if var in marketVars]

            if market != 'cffex' and get_vars != []:
                records = records.append(f(start,get_vars))

        start += datetime.timedelta(days=1)
    return records.reset_index(drop=True)

if __name__ == '__main__':
    df = get_reciept(start='20180711', end=None)
    print(df)



from setuptools import setup, find_packages


long_desc = """
fushare
===============
https://github.com/LowinLi/fushare
Installation
--------------
    pip install fushare

Upgrade
---------------
    pip install fushare --upgrade

"""




setup(
    name='fushare',
    version='1.1.3',
	install_requires = 
		['pandas>=0.23.1',
		'requests>=2.12.4',
		'bs4'],
    description='A utility for fundamentals data of China future',
	packages = ['fushare'],
	package_data={'': ['*.py']},
    long_description=long_desc,
    author='LowinLi',
    author_email='li783170560@126.cn',
    url = 'https://github.com/LowinLi/fushare',
    keywords='Fundmentals Future Data'
)

